package com.workflow.database;

import java.io.Serializable;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;
import org.hibernate.exception.ConstraintViolationException;

import com.workflow.database.beans.Entity;


public class DatabaseUtils {

	@SuppressWarnings("unchecked")
	public static <C> C get(Class<C> C, Serializable id) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		return (C) session.get(C, id);
	}

	/**
	 * Searches the database for an object that matches a given example object.
	 *
	 * @param cl representing the bean class mapped by hibernate
	 * @param exampleObject search example object. An object with matching filled values will be searched.
	 * @return the database object matching the example Object or null if no object is found
	 * @throws HibernateException if there is more than one Object found that matches the exampleObject
	 */
	@SuppressWarnings("unchecked")
	public static <C> C getUnique(Class<C> C, Entity exampleObject) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		return (C) session.createCriteria(C).add(Example.create(exampleObject)).uniqueResult();
	}

	@SuppressWarnings("unchecked")
	public static <C> C getUnique(Class<C> C, Criteria criteria) throws HibernateException {
		return (C) criteria.uniqueResult();
	}

	@SuppressWarnings("unchecked")
	public static <C> List<C> getList(Class<C> C, Entity exampleObject) {
		Session session = WorkflowDAO.getCurrentSession();
		return (List<C>) session.createCriteria(C).add(Example.create(exampleObject)).list();
	}

	@SuppressWarnings("unchecked")
	public static <C> List<C> getbyId(Class<C> C, int id) {
		Session session = WorkflowDAO.getCurrentSession();
		return (List<C>) session.createCriteria(C).add(Expression.eq("id", id)).list();
	}
	
	
	@SuppressWarnings("unchecked")
	public static <C> List<C> getList(Class<C> C, Criteria criteria) throws HibernateException {
		return (List<C>) criteria.list();
	}
	
	@SuppressWarnings("unchecked")
	public static <C> List<C> getList(Class<C> C) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();	
		return (List<C>) session.createCriteria(C).list();
	}
	
	@SuppressWarnings("unchecked")
	public static <C> List<C> getList(Class<C> C, String orderBy) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();	
		return (List<C>) session.createCriteria(C).addOrder(Order.asc(orderBy)).list();
	}

	public static <C> List<C> getAllEntities(Class<C> C) {
		return getAllEntities(C, true);
	}

	@SuppressWarnings("unchecked")
	public static <C> List<C> getAllEntities(Class<C> C, boolean hideDeleted) {
		Session session = WorkflowDAO.getCurrentSession();
		Criteria criteria = session.createCriteria(C);

		return criteria.list();
	}

	/**
	 * Removes an Entity from the DB using autoCommit
	 * 
	 * @param o Entity to delete from the database
	 */
	public static void delete(final Entity o) throws ConstraintViolationException, HibernateException {
		delete(o, true);
	}

	/**
	 * Removes an Entity from the DB
	 * 
	 * @param o Entity to delete from the database
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static void delete(final Entity o, boolean autoCommit) throws ConstraintViolationException, HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try {
			session.delete(o);
			if (autoCommit) WorkflowDAO.commitTransaction();
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}
	
	/**
	 * Removes a list of Serializables from the DB using autoCommit
	 * 
	 * @param o List with Serializables to delete from the database
	 */
	public static void delete(final Iterable<? extends Serializable> o) throws ConstraintViolationException, HibernateException {
		delete(o, true);
	}

	/**
	 * Removes a list of Serializables from the DB
	 * 
	 * @param o List with Serializables to delete from the database
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static void delete(final Iterable<? extends Serializable> o, boolean autoCommit) throws ConstraintViolationException, HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try {
			for (Serializable object: o){
				session.delete(object);				
			}

			if (autoCommit) WorkflowDAO.commitTransaction();
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}

	/**
	 * Add a new Entity to the DB using autoCommit
	 * 
	 * @param o Entity to add to the database
	 */
	public static int add(final Entity o) throws HibernateException {
		return add(o, true);
	}

	/**
	 * Add a new Entity to the DB
	 * 
	 * @param o Entity to add to the database
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static int add(final Entity o, boolean autoCommit) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try{			
			Integer generatedId = (Integer) session.save(o);
			if (autoCommit) WorkflowDAO.commitTransaction();

			return generatedId;
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}

	/**
	 * Add a new Entity to the DB using autoCommit
	 * 
	 * @param o Entity to add to the database
	 */
	public static Integer add(String entityName, final Entity o) throws HibernateException {
		return add(entityName, o, true);
	}

	/**
	 * Add a new Entity to the DB
	 * 
	 * @param o Entity to add to the database
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static Integer add(String entityName, final Entity o, boolean autoCommit) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try{			
			Integer generatedId = (Integer) session.save(entityName, o);
			if (autoCommit) WorkflowDAO.commitTransaction();

			return generatedId;
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}

	/**
	 * Add a list of Serializable to the DB using autoCommit
	 * 
	 * @param o List with Serializable to add to the database
	 */
	public static void add(final Iterable<? extends Serializable> o) throws HibernateException {
		add(o, true);
	}

	/**
	 * Add a list of Serializable to the DB
	 * 
	 * @param o List with Serializable to add to the database
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static void add(final Iterable<? extends Serializable> o, boolean autoCommit) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try {
			for (Serializable object: o){
				session.save(object);				
			}

			if (autoCommit) WorkflowDAO.commitTransaction();
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}

	/**
	 * Add or update an Entity to the DB using autoCommit
	 * 
	 * @param o Entity to add to the database
	 */
	public static void addOrUpdate(final Entity o) throws HibernateException {
		addOrUpdate(o, true);
	}

	/**
	 * Add or update a list of Serializable to/in the DB using autoCommit
	 * 
	 * @param o List with Serializable to add or update to/in the database
	 */
	public static void addOrUpdate(final Iterable<? extends Serializable> o) throws HibernateException {
		addOrUpdate(o, true);
	}

	/**
	 * Add or update a Entity to/in the DB
	 * 
	 * @param o Entity to add or update to/in the database
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static void addOrUpdate(final Entity o, boolean autoCommit) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try {
			session.saveOrUpdate(o);				

			if (autoCommit) WorkflowDAO.commitTransaction();
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}

	/**
	 * Add or update a list of Serializable to the DB
	 * 
	 * @param o List with Serializable to add to the database
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static void addOrUpdate(final Iterable<? extends Serializable> o, boolean autoCommit) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try {
			for (Serializable object: o){
				session.saveOrUpdate(object);				
			}

			if (autoCommit) WorkflowDAO.commitTransaction();
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}

	/**
	 * Update an Entity in the DB using autoCommit
	 * 
	 * @param o Entity to update in the DB
	 */
	public static void update(final Entity o) throws HibernateException {
		update(o, true);
	}

	/**
	 * Update an Entity in the DB
	 * 
	 * @param o Entity to update in the DB
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static void update(final Entity o, boolean autoCommit) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try {
			session.update(o);
			if (autoCommit) WorkflowDAO.commitTransaction();
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}

	
	/**
	 * Update an Entity in the DB using autoCommit
	 * 
	 * @param o Entity to update in the DB
	 */
	public static void merge(final Entity o) throws HibernateException {
		merge(o, true);
	}

	/**
	 * Update an Entity in the DB
	 * 
	 * @param o Entity to update in the DB
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static void merge(final Entity o, boolean autoCommit) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try {
			session.merge(o);
			if (autoCommit) WorkflowDAO.commitTransaction();
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}
	
	/**
	 * Updates a list of Serializable in the DB using autoCommit
	 *
	 * @param o List of Serializable to update in the DB
	 */
	public static void update(final Iterable<? extends Serializable> o) throws HibernateException {
		update(o, true);
	}

	/**
	 * Updates a list of Serializable in the DB
	 *
	 * @param o List of Serializable to update in the DB
	 * @param autoCommit boolean indicating if autoCommit should be used
	 */
	public static void update (final Iterable<? extends Serializable> o, boolean autoCommit) throws HibernateException {
		Session session = WorkflowDAO.getCurrentSession();
		if ((autoCommit && !WorkflowDAO.transactionExists()) || !WorkflowDAO.transactionExists()) {
			WorkflowDAO.getCurrentTransaction();
		}

		try {
			for (Serializable object: o){
				session.update(object);				
			}

			if (autoCommit) WorkflowDAO.commitTransaction();
		} catch (HibernateException he) {
			if (autoCommit) WorkflowDAO.rollBackTransaction();
			throw he;
		}
	}

}
